function [out] = SpaceExToStateflow(varargin)
%SPACEEXTOSTATEFLOW SpaceEx to Stateflow conversion
%
% example call without semantics preservation:
% SpaceExToStateflow('mymodel.xml', 'myconfig.cfg', '--folder', 'myfolder')
% This uses SpaceEx model ..\examples\myfolder\mymodel.xml with myconfig.cfg.
%
% example call with semantics preservation: add '-s' like in the following:
% SpaceExToStateflow('mymodel.xml', 'myconfig.cfg', '--folder', 'myfolder', '-s')
%
% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------

    %Ex: how to instantiate java objects into Matlab
    %add java library for spaceex parsing library (from spaceex2boogie)
    % check if next works with: javaclasspath
    javaaddpath(['.', filesep, 'lib', filesep, 'spaceex-converter.jar']);
    import de.uni_freiburg.informatik.swt.debugprinter.*;
    import de.uni_freiburg.informatik.swt.spaceexboogieprinter.*;
    import de.uni_freiburg.informatik.swt.spaceexxmlprinter.*;
    import de.uni_freiburg.informatik.swt.spaxeexxmlreader.*;
    import de.uni_freiburg.informatik.swt.sxhybridautomaton.*;
    import de.uni_freiburg.informatik.swt.sxlinearapproximation.*;
    
    %
    % add classes and functions subfolders to path
    addpath(genpath('functions'));
    addpath(genpath('classes'));
    % Specify options from the input argument
    [opt_xml_reader, opt_cfg_reader, opt_semantics, opt_display_flow, ...
        opt_display_guard, opt_display_invariant, opt_randgen, ...
        opt_eager_violation, path_name, xml_filename, cfg_filename] = option_SpaceExToStateflow(varargin);
    xml_filepath = [path_name, xml_filename];
    cfg_filepath = [path_name, cfg_filename];
    % add option for using model without .cfg file
    if opt_xml_reader == 1
        try
        % read spaceex XML model file
            xml = xmlread(xml_filepath);  
            [pathstr,name,ext] = fileparts(xml_filepath);  
        catch exception
            %if ~exist('..\examples\xml_filename', 'file')
                disp('The xml file does not exist');
                throw(exception)
            %end 
        end
        if opt_cfg_reader == 1 
            try
                cfg_reader = java.io.FileReader(cfg_filepath);
                % create SpaceexDocuments
                % read both spaceex XML model and configuration files
                xml_cfg = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLReader(xml, cfg_reader);
                doc = xml_cfg.read();
            catch exception
               %if ~exist('..\examples\cfg_filename', 'file')
                    disp('The configuration file does not exist');
                    throw(exception);
               %end
            end
        else
            try
                xml_reader = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLReader(xml);
                %create SpaceexDocuments
                doc = xml_reader.read();
            catch exception
               throw(exception);
            end
        end  
    % create an error message if no xml file found
    else
        throw(MException('ResultChk:BadInput','Argument does not contain xml file, please input the xml file name'));
    end
    
    if opt_randgen == 1
    	output_path = ['.', filesep, 'output_SLSF', filesep];
    else
	output_path = ['.', filesep, 'output', filesep];
    end
    out = doc;
    
    bdclose(name); % close model (uncoditionally, careful in case this is called and open diagrams are not saved!)
    
    % Reference: http://blogs.mathworks.com/seth/2010/01/21/building-models-with-matlab-code/
    % Reference: http://www.mathworks.com/help/stateflow/api/quick-start-for-the-stateflow-api.html
    rt = sfroot;
    m = rt.find('-isa','Simulink.BlockDiagram');
    %chart_ref = m.find('-isa','Stateflow.Chart');
    prev_models = rt.find('-isa','Simulink.BlockDiagram');
    % Create new model, and get current models
    sfnew;
    curr_models = rt.find('-isa','Simulink.BlockDiagram');
    % New model is current models - previous models
    m = setdiff(curr_models, prev_models);
    % Save model file
    %
    sfsave(m.Name, [output_path, name, '.mdl']);
    %
    % Get chart in new model
    ch = m.find('-isa', 'Stateflow.Chart');
    ch.Name = strcat('SF_',name);  
    %Update chart method type to be continuous
    ch.ChartUpdate='CONTINUOUS';
    %ch.ChartUpdate='DISCRETE';
    %offset = 60;
    %Initiate SpaceExBoogiePrinter object
    boogiePrinter = de.uni_freiburg.informatik.swt.spaceexboogieprinter.SpaceExBoogiePrinter(doc);
    %basecompone
    for i_comp = 0 : doc.getComponentCount() - 1
        clear base locations binds transitions;
        base = doc.getComponent(i_comp);
        components(i_comp + 1).base = base;
        % Constant test
        %
        if isa(base, 'BaseComponent')
            mbase = base;
            % Pass all names of constant from SpaceEx into SF
            %
            boogiePrinter.setBase(base);
            boogiePrinter.collectConstants();
            if (opt_semantics)
                [constants] = semanticTranslation(ch, base, ...
                    doc.getInitialStateConditions(), name, opt_eager_violation);
            else
                % translate locations from SpaceEx into SF
                [sF,variables,out_vars,idToLocation,initialConstants] = Location_SpaceExToStateflow(m,ch,base,doc,name,opt_cfg_reader,opt_display_flow,opt_display_invariant);
                % translate transitions from SpaceEx into SF
                Transition_SpaceExToStateflow(sF,ch,base,variables,out_vars,idToLocation,opt_display_guard);
            end
        elseif isa(base, 'NetworkComponent')
            boogiePrinter.setNetWork(base);
            for i_bind = 0 : base.getBindCount() - 1
                binds(i_bind + 1) = base.getBind(i_bind);
                boogiePrinter.setBase(mbase);
                boogiePrinter.setBind(base.getBind(i_bind));
                % Instantiate constantList object and acquire constant
                % values from collectConstantsValue()method
                constantList = boogiePrinter.getConstants();
                boogiePrinter.collectConstantsValue();
                % transfer constant values from SpaceEx into SF

                if (opt_semantics)
                    if (~ (exist('constants', 'var')))
                        constants = cell(0, 2);
                    end
                    % transfer constant values from SpaceEx into SF
                    [constants] = ...
                        semanticTranslationNetwork(constantList, constants);
                else
                    [constantNames] = getConstantValues_SpaceExToStateflow(m,ch,constantList,initialConstants);
                end
            end
        else
            assert(false, 'unknown component type');
        end
    end
    
    if (opt_semantics)
        % initialize constants
        semanticTranslationInitialization(ch, constants);
    end
    % Save model file
    %
    if opt_randgen == 1
        output_path_randgen = ['.', filesep, 'output_SLSF', filesep];
        delete(['.', filesep, 'output', filesep,name,'.mdl']);
        sfsave(m.Name, [output_path_randgen, name, '.mdl']);
    else
        sfsave(m.Name, [output_path, name, '.mdl']);
    end
    %
end
