function [ constants ] = semanticTranslation( chart, base, initCond, modelName, opt_eager_violation )
%SEMANTICTRANSLATION Model translation with semantics preservation.
% Each location instantiates a location pattern for the Stateflow model.
% Similarly, each transition instantiates a transition pattern. First the
% location patterns are created, since (chicken or the egg dilemma) the
% transitions connect the resulting states/junctions.
%
% ------------------------------------------------------------------------------
% author: Christian Schilling
% ------------------------------------------------------------------------------

    % 
    ADD_EPSILON = true;

    % variables, constants, and initial locations of the model
    [variables, constants] = getVariables(base);
    [INITIAL_VARIABLES_STRING, constants, initialLocations] = ...
        parseConfigFile(initCond, constants, ADD_EPSILON);
    
    % position parmameters
    X0 = 50;
    Y0 = 50;
    W = 200;
    H = 150;
    J_H = 0.3 * H;
    
    % new line character (platform independent)
    NL = 10;
    %double(java.lang.System.getProperty('line.separator').char);
    
    % number of locations
    NUM_LOC = base.getLocationCount();
    % number of transitions
    NUM_TRANS = base.getTransitionCount();
    % maximum number of outgoing transitions for any state
    numTransMax = 0;
    
    % constant strings
    STATE_IN_STRING = ...
        ['_in', NL, 'en:', NL, getStoreString(true, variables, NL), NL];
    TRANS_DWELL2LEAVE_STRING = ['[', AuxVarMap.t, ' >= ', AuxVarMap.T, ']'];
    TRANS_LEAVE2DWELL_STRING = ...
        ['[', AuxVarMap.t, ' >= ', AuxVarMap.maxT, ']', NL, ...
        '{', AuxVarMap.stop, ' = true;}'];
    STATE_CHOOSE_STRING = ['_choose', NL, 'en:', NL, ...
        AuxVarMap.T, ' =  chooseT(', AuxVarMap.t, ', ', AuxVarMap.T, ', ', ...
            AuxVarMap.tFail, ', ', AuxVarMap.resets, ', ', ...
            AuxVarMap.maxResets, ', ', AuxVarMap.rand, ');'];
    TRANS_CHOOSES2DWELL_STRING = ...
        ['[', AuxVarMap.resets, ' < ', AuxVarMap.maxResets, ']'];
    TRANS_CHOOSEJ2CHOOSES_STRING = ['[', AuxVarMap.transIdx, ' > 0]', NL, ...
        '{', AuxVarMap.resets, ' = 0;', NL, ...
        AuxVarMap.T, ' = ', AuxVarMap.maxT, ';', NL, ...
        AuxVarMap.trans, ' = ' AuxVarMap.transArray, '(', ...
            AuxVarMap.transIdx, ');', NL, ...
        AuxVarMap.transIdx, ' = ' AuxVarMap.transIdx, ' - 1;}'];
    TRANS_CHOOSES2CHOOSEJ_STRING = ...
        ['[', AuxVarMap.resets, ' == ', AuxVarMap.maxResets, ']'];
    TRANS_CHOOSEJ2DWELL_STRING = ['[', AuxVarMap.transIdx, ' >= 0]', NL, ...
        '{', AuxVarMap.tFail, ' = ', AuxVarMap.t, ';', NL, ...
        getStoreString(false, variables, NL), NL, ...
        AuxVarMap.resets, ' = ' AuxVarMap.resets, ' + 1;}'];
    TRANS_IN2DWELL_STRING = ['[', AuxVarMap.transIdx, ' == 0]', NL, ...
        '{', AuxVarMap.T, ' = ' AuxVarMap.maxT, ';', NL, ...
        AuxVarMap.transIdx, ' = -1;', NL, AuxVarMap.trans, ' = 0;}'];
    TRANS_BACKTRACK2DWELL_STRING = ['[', AuxVarMap.transIdx, ' == -1]', NL, ...
        '{', AuxVarMap.stop, ' = true;}'];
    TRANS_LEAVE2TRANS_STRING = ['[', AuxVarMap.trans, ' == '];
    
    % mapping from locations to entry states
    loc2states = cell(NUM_LOC, 2);
    
    % mapping from locations to outgoing transitions
    loc2trans = cell(NUM_LOC, 1);
    for iTrans = 0 : NUM_TRANS - 1
        trans = base.getTransition(iTrans);
        source = trans.getSource();
        loc2trans{source} = [loc2trans{source}, trans];
    end
    
    % iterate over all locations
    for iLoc = 0 : NUM_LOC - 1
        loc = base.getLocation(iLoc);
        locId = loc.getId();
        locName = char(loc.getName());
        outTransitions = loc2trans{locId};
        NUM_OUT_TRANS = size(outTransitions, 1);
        numTransMax = max(numTransMax, NUM_OUT_TRANS);
        
        % add entry state:
        % - stores variables to temporary variables
        % - constructs new transition array
        % - resets transition array index
        state_in = Stateflow.State(chart);
        state_in.LabelString = [locName, STATE_IN_STRING, ...
            AuxVarMap.loc, ' = ', num2str(locId), ';', NL, ...
            getTransitionArrayString(NUM_OUT_TRANS, NL)];
        state_in.Position = [X0 + (2 * iLoc * W), Y0, W, H];
        loc2states{locId, 1} = state_in;
        
        % add junction for choosing transitions:
        % - goes to choose-state when there are transitions left
        % - goes to dwell-state when there are no transitions left
        junction_in2dwell = Stateflow.Junction(chart);
        junction_in2dwell.Position.Center = ...
            [state_in.Position(1) + (W / 2), state_in.Position(2) + H + J_H];
        
        % add transition 'entry state -> transition choosing junction'
        transition = addTransition(chart, state_in, junction_in2dwell);
        transition.SourceOClock = 6;
        transition.DestinationOClock = 0;
        
        % determine next state's position depending on outgoing transitions
        if (NUM_OUT_TRANS > 0)
            dwellY = 4*H;
        else
            dwellY = 2*H;
        end
        
        % add dwelling state:
        % - dwell until either the dwelling time is reached and a transition is
        % enabled or until the invariant is violated
        state_dwell = Stateflow.State(chart);
        state_dwell.LabelString = getFlowString(locName, loc.getFlow(), NL);
        state_dwell.Position = state_in.Position + [0, dwellY, 0, 0];

        % add junction for backtracking
        junction_dwell2choose = Stateflow.Junction(chart);
        junction_dwell2choose.Position.Center = ...
            [state_dwell.Position(1) + 0.9 * W, ...
            state_dwell.Position(2) - H + J_H];
        
        % add junction for leaving dwell-state
        junction_dwell2next = Stateflow.Junction(chart);
        junction_dwell2next.Position.Center = ...
            [state_dwell.Position(1) + (W / 2), ...
            state_dwell.Position(2) + H + J_H];
        loc2states{locId, 2} = junction_dwell2next;
        
        % add transition 'dwelling state -> leaving junction'
        % = dwelling successful
        transition = addTransition(chart, state_dwell, junction_dwell2next);
        transition.LabelString = TRANS_DWELL2LEAVE_STRING;
        transition.SourceOClock = 6;
        transition.DestinationOClock = 0;
        midPt = transition.MidPoint;
        transition.LabelPosition(1) = midPt(1) + 5;
        transition.LabelPosition(2) = midPt(2) - 5;
        
        % add transition 'dwelling state -> backtracking junction'
        % = invariant violation
        transition = addTransition(chart, state_dwell, junction_dwell2choose);
        transition.LabelString = ...
            ['[~(', getInvariantString(loc.getInvariant(), ADD_EPSILON), ')]'];
        transition.SourceOClock = 1;
        transition.DestinationOClock = 6;
        transition.MidPoint(1) = transition.MidPoint(1) - 10;
        transition.DestinationEndPoint(1) = ...
            transition.DestinationEndPoint(1) - 5;
        midPt = transition.MidPoint;
        transition.LabelPosition(1) = ...
            midPt(1) - transition.LabelPosition(3) - 5;
        transition.LabelPosition(2) = midPt(2) - 5;
        if (opt_eager_violation)
            % execute this transition first of option is set accordingly
            transition.ExecutionOrder = 1;
        end
        
        % add transition 'leaving junction -> dwelling state'
        % = maximum simulation time passed: stop simulation
        transition = addTransition(chart, junction_dwell2next, state_dwell);
        transition.LabelString = TRANS_LEAVE2DWELL_STRING;
        transition.SourceOClock = 9;
        transition.DestinationOClock = 7;
        midPt = transition.MidPoint;
        transition.LabelPosition(1) = ...
            midPt(1) - transition.LabelPosition(3) - 5;
        transition.LabelPosition(2) = midPt(2) - 5;
        
        % extra handling in case there are outgoing transitions
        if (NUM_OUT_TRANS > 0)
            % add dwelling time choosing state:
            % - restores the variables (backtracking)
            % - chooses the next dwelling time
            state_choose = Stateflow.State(chart);
            state_choose.LabelString = [locName, STATE_CHOOSE_STRING];
            state_choose.Position = state_in.Position + [0, 2*H, 0, 0];
            
            % add transition 'choosing state -> dwelling state'
            % = try the next time choice
            transition = addTransition(chart, state_choose, state_dwell);
            transition.LabelString = TRANS_CHOOSES2DWELL_STRING;
            transition.SourceOClock = 7;
            transition.DestinationOClock = 11;
            midPt = transition.MidPoint;
            transition.LabelPosition(1) = ...
                midPt(1) - (transition.LabelPosition(3) / 2);
            transition.LabelPosition(2) = midPt(2) - 10;
            
            % add transition 'transition choosing junction -> choosing state'
            % = try the next (original) transition
            transition = addTransition(chart, junction_in2dwell, state_choose);
            transition.LabelString = TRANS_CHOOSEJ2CHOOSES_STRING;
            transition.SourceOClock = 6;
            transition.DestinationOClock = 0;
            transition.MidPoint(1) = transition.MidPoint(1) + 10;
            transition.DestinationEndPoint(1) = ...
                transition.DestinationEndPoint(1) + 5;
            midPt = transition.MidPoint;
            transition.LabelPosition(1) = midPt(1) + 5;
            transition.LabelPosition(2) = midPt(2) - 25;
            
            % add transition 'choosing state -> transition choosing junction'
            % = backtracking (to choose the next transition)
            transition = addTransition(chart, state_choose, junction_in2dwell);
            transition.LabelString = TRANS_CHOOSES2CHOOSEJ_STRING;
            transition.SourceOClock = 0;
            transition.DestinationOClock = 6;
            transition.MidPoint(1) = transition.MidPoint(1) - 10;
            transition.DestinationEndPoint(1) = ...
                transition.DestinationEndPoint(1) - 5;
            midPt = transition.MidPoint;
            transition.LabelPosition(1) = ...
                midPt(1) - transition.LabelPosition(3) - 5;
            transition.LabelPosition(2) = midPt(2) - 10;
            
            % add transition 'time choosing junction -> dwelling state'
            % = choose new dwelling time
            transition = ...
                addTransition(chart, junction_dwell2choose, state_choose);
            transition.LabelString = TRANS_CHOOSEJ2DWELL_STRING;
            transition.SourceOClock = 0;
            transition.DestinationOClock = 5;
            midPt = transition.MidPoint;
            transition.LabelPosition(1) = midPt(1) + 5;
            transition.LabelPosition(2) = midPt(2) - 20;
        end
        
        % add transition 'entry junction -> dwelling state'
        % = no transitions are left
        transition = addTransition(chart, junction_in2dwell, state_dwell);
        transition.LabelString = TRANS_IN2DWELL_STRING;
        transition.DestinationOClock = 9;
        transition.DestinationEndPoint(1) = state_dwell.Position(1);
        transition.DestinationEndPoint(2) = state_dwell.Position(2) + (H / 2);
        transition.SourceOClock = 9;
        transition.MidPoint(1) = transition.MidPoint(1) + 30;
        midPt = transition.MidPoint;
        transition.LabelPosition(1) = ...
            midPt(1) - transition.LabelPosition(3) - 5;
        transition.LabelPosition(2) = midPt(2) - 25;
        
        % add transition 'backtracking junction -> dwelling state'
        % = deadlock: stop simulation
        transition = addTransition(chart, junction_dwell2choose, state_dwell);
        transition.LabelString = TRANS_BACKTRACK2DWELL_STRING;
        transition.SourceOClock = 6;
        transition.DestinationOClock = 0;
        transition.MidPoint(1) = transition.MidPoint(1) + 10;
        transition.DestinationEndPoint(1) = ...
            transition.DestinationEndPoint(1) + 5;
        midPt = transition.MidPoint;
        transition.LabelPosition(1) = midPt(1) + 5;
        transition.LabelPosition(2) = midPt(2) - 10;
    end
    
    % constant strings relying on the maximum number of outgoing transitions
    arrayString = '[1';
    for i = 2 : numTransMax
        arrayString = [arrayString, ', ', num2str(i)];
    end
    INITIAL_STRING = ['{', INITIAL_VARIABLES_STRING, NL, ...
        AuxVarMap.t, ' = 0;', NL, ...
        AuxVarMap.tOut, ' = 0;', NL, ...
        AuxVarMap.backtrack, ' = 1;', NL, ...
        AuxVarMap.stop, ' = false;', NL, ...
        AuxVarMap.loc, ' = 0;', NL, ...
        AuxVarMap.transArray, ' = ', arrayString, '];}'];
    
    % different case handling for initial locations
    INIT_LOC_NUM = size(initialLocations, 2);
    if (INIT_LOC_NUM == 0)
        % no initial transition
        throw(MException('Translation:noInitLoc', 'no initial location found'));
    elseif (INIT_LOC_NUM == 1)
        % one initial transition
        state_init = [];
    else
        % several initial transitions
        
        % add initial state
        state_init = Stateflow.State(chart);
        state_init.LabelString = ['initial', NL, 'en:', NL, ...
            AuxVarMap.init, ' = round(random(', AuxVarMap.rand, ', 1, ', ...
            num2str(INIT_LOC_NUM), '));'];
        state_init.Position = [0, -10, 100, 50];
        
        % add initial transition
        transition = Stateflow.Transition(chart);
        transition.Destination = state_init;
        transition.DestinationOClock = 0;
        srcX = state_init.Position(1) + state_init.Position(3) / 2;
        transition.SourceEndPoint = [srcX, state_init.Position(2) - 20];
        transition.SourceOClock = 6;
        transition.LabelString = INITIAL_STRING;
        transition.LabelPosition(1) = srcX + 5;
        transition.LabelPosition(2) = ...
            transition.DestinationEndPoint(2) - transition.LabelPosition(4);
        
        % set index
        initIdx = 0;
    end
    
    % For adding the original transitions, we need to have all states created.
    % So we start a new loop here.
    for iLoc = 0 : NUM_LOC - 1
        loc = base.getLocation(iLoc);
        locId = loc.getId();
        locName = char(loc.getName());
        outTransitions = loc2trans{locId};
        NUM_OUT_TRANS = size(outTransitions, 1);
        junction_dwell2next = loc2states{locId, 2};
        juncX = junction_dwell2next.Position.Center;
        juncY = juncX(2);
        juncX = juncX(1);
        
        % add initial transition (acquired from configuration file)
        for i = 1 : INIT_LOC_NUM
            if (strcmp(initialLocations{i}, locName))
                state_in = loc2states{locId, 1};
                transition = Stateflow.Transition(chart);
                transition.Destination = state_in;
                transition.DestinationOClock = 0;
                if (isempty(state_init))
                    % the only initial transition
                    srcX = state_in.Position(1) + W / 2;
                    transition.SourceEndPoint = [srcX, 0];
                    transition.SourceOClock = 6;
                    transition.LabelString = INITIAL_STRING;
                    transition.LabelPosition(1) = srcX + 5;
                    transition.LabelPosition(2) = ...
                        Y0 - transition.LabelPosition(4);
                else
                    % only connect to initial state
                    transition.Source = state_init;
                    transition.SourceOClock = 3;
                    initIdx = initIdx + 1;
                    transition.LabelString = ...
                        ['[', AuxVarMap.init, ' == ' num2str(initIdx), ']'];
                    endPt = transition.DestinationEndPoint;
                    transition.LabelPosition(1) = endPt(1) + 5;
                    transition.LabelPosition(2) = endPt(2) - 20;
                end
                break;
            end
        end
        
        % add a junction for each (original) transition
        for iTrans = 1 : NUM_OUT_TRANS
            % add junction for taking (original) transition
            junction_trans = Stateflow.Junction(chart);
            junction_trans.Position.Center = [juncX, juncY + (iTrans * J_H)];
            
            % add transition 'leaving junction -> transition junction'
            transition = ...
                addTransition(chart, junction_dwell2next, junction_trans);
            transition.LabelString = ...
                [TRANS_LEAVE2TRANS_STRING, num2str(iTrans), ']'];
            if (mod(iTrans, 2) == 0)
                % go right
                transition.SourceOClock = 3;
                transition.DestinationOClock = 3;
                midPt = transition.MidPoint;
                transition.LabelPosition(1) = midPt(1) + 5;
                transition.LabelPosition(2) = midPt(2) - 5;
            else
                % go left
                transition.SourceOClock = 9;
                transition.DestinationOClock = 9;
                midPt = transition.MidPoint;
                transition.LabelPosition(1) = ...
                    midPt(1) - transition.LabelPosition(3) - 5;
                transition.LabelPosition(2) = midPt(2) - 5;
            end
            
            % add transition 'transition junction -> entry state'
            % = original transition
            trans = outTransitions(iTrans);
            targetId = trans.getTarget;
            targetLoc = base.getLocationByID(targetId);
            state_in = loc2states{targetId, 1};
            transition = addTransition(chart, junction_trans, state_in);
            transition.LabelString = ...
                getTransitionString(targetLoc, trans, NL, ADD_EPSILON);
            
            if (state_in.Position(1) > juncX)
                % go right
                transition.DestinationOClock = 9;
                transition.SourceOClock = 3;
                transition.DestinationEndPoint(1) = state_in.Position(1);
            elseif (state_in.Position(1) + W > juncX)
                % self-loop
                transition.DestinationOClock = 9;
                transition.SourceOClock = 9;
                transition.DestinationEndPoint(1) = state_in.Position(1);
            else
                % go left
                transition.DestinationOClock = 3;
                transition.SourceOClock = 9;
                transition.DestinationEndPoint(1) = state_in.Position(1) + W;
            end
            transition.DestinationEndPoint(2) = Y0 + (H / 2);
            midPt = transition.MidPoint;
            transition.LabelPosition(1) = midPt(1);
            transition.LabelPosition(2) = midPt(2);
        end
    end
    
    % add auxiliary variables
    vars = AuxVarMap.elemCell();
    for i = 1 : length(vars)
        var = vars(i, :);
        sF_vars = Stateflow.Data(chart);
        sF_vars.Name = var{1};
        switch (var{2})
            case 'l'
                sF_vars.Scope = 'Local';
            case 'o'
                sF_vars.Scope = 'Output';
            case 'p'
                sF_vars.Scope = 'Parameter';
            case 'i'
                sF_vars.Scope = 'Input';
            otherwise
                assert(false, 'Illegal variable definition.');
        end
        sF_vars.Props.Complexity = 'Off';
        sF_vars.Props.Array.Size = '';
        switch (var{3})
            case 'c'
                sF_vars.Update = 'CONTINUOUS';
            case 'd'
                sF_vars.Update = 'DISCRETE';
                varname = var{4};
                switch varname
                    case {'d', 'i'}
                        % integer type is badly supported => use double
                        sF_vars.DataType = 'double';
                    case 'a'
                        sF_vars.DataType = 'double';
                        switch var{1}
                            case {AuxVarMap.transArray, AuxVarMap.randArray}
                                sF_vars.Props.Array.Size = ...
                                    ['[1, ', num2str(numTransMax), ']'];
                                
                            otherwise
                                assert(false, 'Unknown array variable.');
                        end
                    otherwise
                        sF_vars.DataType = varname;
                end
            otherwise
                assert(false, 'Illegal variable definition.');
        end
    end
    
    chartName = chart.Name;
    
    % add stop block
    add_block('built-in/Stop', [modelName '/Stop'], ...
        'Position', [450, 0, 500, 50]);
    line = ...
        add_line(modelName, [chartName, '/1'], 'Stop/1', 'autorouting', 'on');
    set_param(line, 'Name', 'stop');
    
    % add random number block
    addRandomBlock([modelName '/RandNum'], [25, 125, 75, 175], '0', '1');
    line = add_line(modelName, 'RandNum/1', [chartName, '/1'], ...
        'autorouting', 'on');
    set_param(line, 'Name', 'randNum');
    
    % add random array block
    total = num2str(numTransMax + 1);
    minString = '[1';
    minAddString = ', 1';
    maxString = ['[', total];
    maxAddString = [', ', total];
    for i = 2 : numTransMax
        minString = [minString, minAddString];
        maxString = [maxString, maxAddString];
    end
    minString = [minString, ']'];
    maxString = [maxString, ']'];
    addRandomBlock([modelName '/RandArr'], [25, 200, 75, 250], minString, ...
        maxString);
    line = add_line(modelName, 'RandArr/1', [chartName, '/2'], ...
        'autorouting', 'on');
    set_param(line, 'Name', 'randArr');
    
    % number of auxiliary variables linked to scope
    SCOPE_OFFSET = 3;
    
    % add scope block
    add_block('built-in/Scope', [modelName '/Scope'], ...
        'NumInputPorts', num2str(size(variables, 1) + SCOPE_OFFSET), ...
        'Position', [450, 75, 500, 150], ...
        'SaveToWorkspace', 'on', ...
        'SaveName', 'ScopeData', ...
        'DataFormat', 'Structure', ...
        'LimitDataPoints', 'off');
    
    % link time variable and backtracking information to scope
    linkToScope(modelName, chartName, 2, 1, 'time');
    linkToScope(modelName, chartName, 3, 2, 'backtrack');
    linkToScope(modelName, chartName, 4, 3, 'location');
    
    % create local and output variables
    for i = 1: size(variables, 1)
        variable = variables{i};
        
        % original variable
        var = Stateflow.Data(chart);
        var.Name = [AuxVarMap.PREFIX_VARIABLE, variable];
        var.Scope = 'Local';
        var.Update = 'CONTINUOUS';
        var.Props.Array.Size = '';
        
        % temporary variable
        var = Stateflow.Data(chart);
        var.Name = [AuxVarMap.PREFIX_ASSIGNMENT , variable];
        var.Scope = 'Local';
        var.Update = 'DISCRETE';
        
        % output variable
        var = Stateflow.Data(chart);
        var.Name = [AuxVarMap.PREFIX_OUTPUT, variable];
        var.Scope = 'Output';
        var.Update = 'DISCRETE';
        
        % link the output to scope
        linkToScope(modelName, chartName, i + SCOPE_OFFSET + 1, ...
            i + SCOPE_OFFSET, variable);
    end
    
    % set simulation time to infinity
    set_param(modelName, 'StopTime', 'inf');
end


function [ variables, constants ] = getVariables( base )
% extracts variables and constants from model
    totalParams = base.getParamCount();
    variables = cell(totalParams, 1);
    varCtr = 0;
    constants = cell(totalParams, 2);
    constCtr = 0;
    for i = 1 : totalParams
        param = base.getParam(i - 1);
        type = char(param.getType());
        if (strcmp(type, 'REAL'))
            name = char(param.getName());
            switch (char(param.getDynamics))
                case 'ANY'
                    varCtr = varCtr + 1;
                    variables{varCtr} = name;
                case 'CONST'
                    constCtr = constCtr + 1;
                    constants{constCtr, 1} = name;
                    constants{constCtr, 2} = '';
                otherwise
                    assert(false, 'Unknown dynamics!');
            end
        else
            assert(strcmp(type, 'LABEL'), 'Unknown type!');
        end
    end
    variables = variables(1: varCtr);
    constants = constants(1: constCtr, :);
end


function [ initialVariables, constants, initialLocations ] = parseConfigFile( initial, constants, addEpsilon )
% parses the initial conditions from the config file
% contains:
% - initial values for all constants
% may contain:
% - initial values for some variables
% - initial location(s)
    initial = convertExpression(initial, EEqualityHandling.doubleEq, false, ...
        addEpsilon);
    initialVariables = '';
    initialLocations = cell(1, 1);
    numLocations = 0;
    
    conjuncts = regexp(initial, '&&', 'split');
    for i = 1 : length(conjuncts)
        conjunct = conjuncts{i};
        % TODO handle nondeterministic condition
        parts = regexp(conjunct, '==', 'split');
        assert(length(parts) == 2, 'Unknown condition!');
        lhs = char(parts(1));
        rhs = char(parts(2));

        % check different cases
        if (isempty(regexp(lhs, 'loc\(\w*\)', 'match')))
            % scan for constants
            found = false;
            for j = 1 : size(constants, 1)
                constant = constants(j, 1);
                if (strcmp(lhs, constant))
                    % specification of constant
                    constants{j, 2} = rhs;
                    found = true;
                    break;
                end
            end
            
            % specification of variable, simply concatenate
            if (~ found)
                initialVariables = [initialVariables, ...
                    AuxVarMap.PREFIX_VARIABLE, lhs, ' = ', rhs, '; '];
            end
        else
            % specification of initial location
            % TODO Should we be able to find initial locations when none is
            % specified? This requires reasoning about the invariants.
            numLocations = numLocations + 1;
            initialLocations{numLocations} = rhs;
        end
    end
end


function [ flow ] = getFlowString( locName, flow, NL )
% returns the string for the flow, including the output variable assignments
    flow = convertExpression(flow, EEqualityHandling.singleEq, true, false);
    conjuncts = regexp(flow, '&&', 'split');
    flow = '';
    output = '';
    for i = 1 : length(conjuncts)
        conjunct = conjuncts{i};
        parts = regexp(conjunct, '''=', 'split');
        lhs = char(parts{1});
        rhs = char(parts{2});
        flow = [flow, lhs, '_dot = ', rhs, ';', NL];
        % need to replace variable prefix with output prefix here (a bit ugly)
        output = [output, AuxVarMap.PREFIX_OUTPUT, ...
            lhs(length(AuxVarMap.PREFIX_VARIABLE) + 1 : length(lhs)), ...
            ' = ', lhs, ';', NL];
    end
    
    tString = AuxVarMap.t;
    flow = [locName, '_dwell', NL, 'du:', NL, ...
            flow, tString, '_dot = 1;', NL, ...
            output, AuxVarMap.tOut, ' = ', tString, ';'];
end


function [ str ] = getTransitionString( targetLoc, transition, NL, addEpsilon )
% returns the string for an original transition, containing guard and assignment
    % guard string
    guardString = convertExpression(...
        transition.getGuard(), EEqualityHandling.interval, true, addEpsilon);
    
    % assignment string (make it concurrent)
    assignment = transition.getAssignment();
    assignmentString = '';
    if ~isempty(assignment)
        tmpVars = '';
        conjuncts = strsplit(convertExpression(...
            assignment, EEqualityHandling.singleEq, true, false), '&&');
        startIdx = length(AuxVarMap.PREFIX_VARIABLE) + 1;
        for i = 1: length(conjuncts)
            eqSplit = strsplit(conjuncts{i}, '''=');
            assert(length(eqSplit) == 2, ...
                'conjunct should contain exactly one equality.');
            lhsMod = eqSplit{1};
            lhsOrig = lhsMod((startIdx) : length(lhsMod));
            rhsMod = eqSplit{2};
            if (strcmp(lhsMod, rhsMod))
                % identity update, ignore this
                continue;
            end
            
            % The original assignment is:
            %   x = e;
            % But to make assignments concurrent, we write:
            %   x_tmp = e;
            % and later add
            %   x = x_tmp;
            assignmentString = [assignmentString, ...
                    AuxVarMap.PREFIX_ASSIGNMENT, lhsOrig, ' = ', rhsMod, ';'];
            tmpVars = [tmpVars, lhsMod, ' = ', AuxVarMap.PREFIX_ASSIGNMENT, ...
                lhsOrig, '; '];
        end
        if (~ isempty(tmpVars))
            assignmentString = [assignmentString, 10, tmpVars];
        end
    end
    
    % invariant formula
    invariant = targetLoc.getInvariant();
    
    if (isempty(invariant))
        if (isempty(guardString))
            % neither guard nor invariant
            if (~ isempty(assignmentString))
                str = ['{', assignmentString, '}'];
            else
                str = '';
            end
        else
            % only guard
            str = ['[', guardString, ']'];
            if (~ isempty(assignmentString))
                str = [str, NL, '{', assignmentString, '}']; 
            end
        end
    else
        if (isempty(guardString))
            % only invariant
            str = '[';
        else
            % guard and invariant
            str = ['[', guardString, ' && ...', NL];
        end
        if (isempty(assignmentString))
            % guard and invariant
            str = [str, getInvariantString(invariant, addEpsilon), ']'];
        else
            % guard and wp(invariant, assignment)
            str = [str, getWpString(invariant, assignment, addEpsilon), ...
                    ']', NL, ...
                '{', assignmentString, '}'];
        end
    end
end


function [ transition ] = addTransition( chart, src, dest )
% adds a new transition for Simulink
    transition = Stateflow.Transition(chart);
    transition.Source = src;
    transition.Destination = dest;
end


function [ str ] = getStoreString( store, variables, NL )
% returns the string for storing/restoring variables
    if (store)
        % store
        left = AuxVarMap.PREFIX_ASSIGNMENT;
        right = AuxVarMap.PREFIX_VARIABLE;
    else
        % restore
        left = AuxVarMap.PREFIX_VARIABLE;
        right = AuxVarMap.PREFIX_ASSIGNMENT;
    end
    
    str = '';
    
    % normal variables
    for i = 1 : size(variables, 1)
        var = variables{i};
        str = [str, left, var, ' = ', right, var, ';', NL];
    end
    
    % output variables
    for i = 1 : size(variables, 1)
        var = variables{i};
        str = [str, AuxVarMap.PREFIX_OUTPUT, var, ' = ', ...
            AuxVarMap.PREFIX_VARIABLE, var, ';', NL];
    end
    
    if (store)
        % store
        str = [str, AuxVarMap.tTmp, ' = ', AuxVarMap.t, ';', NL, ...
            AuxVarMap.tOut, ' = ', AuxVarMap.t, ';', NL, ...
            AuxVarMap.tFail, ' = ', AuxVarMap.maxT, ';', NL, ...
            AuxVarMap.backtrack, ' = ', AuxVarMap.backtrack, ' - 1;'];
    else
        % restore
        str = [str, AuxVarMap.t, ' = ', AuxVarMap.tTmp, ';', NL, ...
            AuxVarMap.tOut, ' = ', AuxVarMap.t, ';', NL, ...
            AuxVarMap.backtrack, ' = ', AuxVarMap.backtrack, ' + 1;'];
    end
end


function [ str ] = getTransitionArrayString( size, NL )
% returns the string for constructing the transitions array
    if (size > 0)
        sizeStr = num2str(size);
        str = [AuxVarMap.transArray, ' = permuteA(', AuxVarMap.transArray, ...
            ', ', sizeStr, ', ', AuxVarMap.randArray ');', NL, ...
            AuxVarMap.transIdx, ' = ', sizeStr, ';'];
    else
        str = [AuxVarMap.transIdx, ' = 0;'];
    end
end


function [ str ] = getInvariantString( invariant, addEpsilon )
% returns the string for the invariant
    str = convertExpression(invariant, EEqualityHandling.interval, ...
        true, addEpsilon);
end


function [ str ] = getWpString( invariant, assignment, addEpsilon )
% computes the weakest precondition of an invariant and an assignment
%
% Since we are only dealing with assignments, wp(I, X := E) is just
% I[E/X], i.e., we substitute every variable which is assigned by the
% assigned expression.
    prefix = 'de.uni_freiburg.informatik.swt.sxhybridautomaton.formula.';
    assert(~isempty(assignment) && isa(assignment, [prefix, 'Operation']), ...
        'Assignment must be a nonempty formula.');
    stack = cell(5, 1);
    stack{1} = assignment;
    stackDepth = 1;
    var2replace = cell(1, 2);
    var2replaceSize = 0;
    while (stackDepth > 0)
        formula = stack{stackDepth};
        assert (isa(formula, [prefix, 'Operation']), ...
            'The formula must be an operation.');
        op = char(formula.getOperator.toString);
        switch op
            case 'EQUAL'
                var2replaceSize = var2replaceSize + 1;
                var = char(formula.getLeft.get);
                var2replace{var2replaceSize, 1} = var;
                var2replace{var2replaceSize, 2} = formula.getRight;
                stackDepth = stackDepth - 1;
            case 'AND'
                stack{stackDepth} = formula.getLeft;
                stackDepth = stackDepth + 1;
                stack{stackDepth} = formula.getRight;
            otherwise
                assert(false, 'Unsupported assignment formula.');
        end
    end

    wp = compute_wp_helper(invariant, var2replace);
    str = convertExpression(wp, EEqualityHandling.interval, true, addEpsilon);
end


function [ outTree ] = compute_wp_helper( inTree, var2replace )
% recursive helper method for computing the weakest precondition
    prefix = 'de.uni_freiburg.informatik.swt.sxhybridautomaton.formula.';
    outTree = inTree;
    if isa(inTree, [prefix, 'Variable'])
        var = char(inTree.get);
        for j = 1 : size(var2replace, 1)
            if (strcmp(var2replace{j, 1}, var))
                outTree = var2replace{j, 2};
                break;
            end
        end
    elseif isa(inTree, [prefix, 'Operation'])
        lhs = compute_wp_helper(inTree.getLeft(), var2replace);
        rhs = compute_wp_helper(inTree.getRight(), var2replace);
        outTree = ...
            de.uni_freiburg.informatik.swt.sxhybridautomaton.formula.Operation(...
            lhs, inTree.getOperator(), rhs);
    end
end


function addRandomBlock( name, pos, minString, maxString )
    add_block('dspsrcs4/Random Source', name, 'Position', pos, ...
        'SrcType', 'Uniform', 'SampMode', 'Continuous', 'MinVal', minString, ...
        'MaxVal', maxString, ...
        'RepMode', 'Specify seed', 'rawSeed', AuxVarMap.seed);
end


function linkToScope( modelName, chartName, outPort, inPort, lineName )
% connects the chart with the scope wrt. given ports
    line = add_line(modelName, [chartName, '/', num2str(outPort)], ...
        ['Scope/', num2str(inPort)], 'autorouting', 'on');
    set_param(line, 'Name', lineName);
end


function [ str ] = convertExpression( formula, equality_handling, addVariablePrefix, addEpsilon )
% converts a SpaceEx formula to a string
% TODO atm mostly copied from Luan, rewrite to own needs
    if (isempty(formula))
        assert(false, 'empty formula');
    end
    
    epsString = '';
    prefix = 'de.uni_freiburg.informatik.swt.sxhybridautomaton.formula.';

    if isa(formula, [prefix, 'Variable'])
        if (addVariablePrefix)
            str = [AuxVarMap.PREFIX_VARIABLE, char(formula.get)];
        else
            str = char(formula.get);
        end
        if formula.isPrimed
            str = strcat(str, '''');
        end
    elseif isa(formula, [prefix, 'Constant'])
        str = char(formula);
    elseif  isa(formula, [prefix, 'Operation'])    
        left = formula.getLeft();
        right = formula.getRight();
        op = char(formula.getOperator.toString);
        switch op % TODO: case sensitivity?
            case 'EQUAL'
                switch (equality_handling)
                    case EEqualityHandling.singleEq
                        opconv = '=';
                    case EEqualityHandling.doubleEq
                        opconv = '==';
                    case EEqualityHandling.interval
                        % interval (lhs - eps <= rhs && lhs + eps >= rhs)
                        lhs = convertExpression(left, ...
                            EEqualityHandling.singleEq, ...
                            addVariablePrefix, addEpsilon);
                        rhs = convertExpression(right, ...
                            EEqualityHandling.singleEq, ...
                            addVariablePrefix, addEpsilon);
                        str = strcat('(', lhs , ' - ', AuxVarMap.eps, ...
                            ' <= ', rhs, ' && ', lhs , ' + ', ...
                            AuxVarMap.eps, ' >= ', rhs ,')');
                        return;
                end
            case 'NOTEEQUAL'
                opconv = '~=';
            case 'LESS'
                opconv = '<';
                if (addEpsilon)
                    epsString = [' + ', AuxVarMap.eps];
                end
            case 'GREATER'
                opconv = '>';
                if (addEpsilon)
                    epsString = [' - ', AuxVarMap.eps];
                end
            case 'LESSEQUAL'
                opconv = '<=';
                if (addEpsilon)
                    epsString = [' + ', AuxVarMap.eps];
                end
            case 'GREATEREQUAL'
                opconv = '>=';
                if (addEpsilon)
                    epsString = [' - ', AuxVarMap.eps];
                end
            case 'MULTIPLY'
                opconv = '*';
            case 'DIVIDE'
                opconv = '/';
            case 'ADD'
                opconv = '+';
            case 'SUBTRACT'
                opconv = '-';
            case 'AND'
                opconv = '&&';
            case 'OR'
                opconv = '||';
            otherwise
                opconv = op;
        end

        % possibly add parentheses (assuming the priority is the same in
        % Stateflow as for SpaceEx)
        parent = formula.getParent();
        if (~ isempty(parent) && ...
                (formula.getOperator.getPriority(parent.getOperator()) < ...
                formula.getOperator.getPriority(formula.getOperator())))
            leftParenthesis = '(';
            rightParenthesis = ')';
        else
            leftParenthesis = '';
            rightParenthesis = '';
        end
        
        % construct result string recursively
        str = strcat(leftParenthesis, convertExpression(left, ...
            equality_handling, addVariablePrefix, addEpsilon), ' ', ...
            opconv, ' ', convertExpression(right, equality_handling, ...
            addVariablePrefix, addEpsilon), epsString, rightParenthesis);
    else
        assert(false, 'unkonwn formula');
    end
end