function simulationLoop(model, iterations, maxTime, maxResets, epsilon, filter, seed_option, plot_option)
%SIMULATIONLOOP Iteratively simulates single trajectories and plots them all
%
% example usage: simulationLoop('mymodel', 10, 20, 5, 0.1, {}, 1, 1)
%
% seed_option:
% 1 = always shuffle the random seed in each iteration
% 2 = make results reproducible
%
% plot_option:
% 0 = one plot
% 1 = subplots for each variable
%
% ------------------------------------------------------------------------------
% authors: Christian Schilling, Taylor T. Johnson
% ------------------------------------------------------------------------------

    % add classes and functions subfolders to path
    addpath(genpath('functions'));
    addpath(genpath('classes'));
    
    % open model
%     open_system(['./output/' model]);
    load_system(['./output/' model])
    
    % plot_option enum cases
    plot_phase = 2; % show phase portrait
    
    opt_config_change = 0;
    opt_time = 1;
    
    if opt_time
        tic;
    end
    
    % set standard values
    if ((nargin < 8) || (plot_option == -1))
        plot_option = 1;
    end
    if ((nargin < 7) || (seed_option == -1))
        seed_option = 1;
    end
    if (nargin < 6)
        filter = cell(1, 0);
    end
    if ((nargin < 5) || (epsilon == -1))
        epsilon = 0.0001;
    end
    if ((nargin < 4) || (maxResets == -1))
        maxResets = 3;
    end
    if ((nargin < 3) || (maxTime == -1))
        % try to load maximum simulation time from a model constant
        rt = sfroot;
        modelInstance = rt.find('-isa', 'Simulink.BlockDiagram');
        chart = modelInstance.find('-isa', 'Stateflow.Chart', ...
            '-and', 'Name', ['SF_', model]);
        for i = 1 : 4
            switch i
                case 1
                    x = chart.find('Name', [AuxVarMap.PREFIX_VARIABLE, 'Tmax']);
                case 2
                    x = chart.find('Name', [AuxVarMap.PREFIX_VARIABLE, 'maxT']);
                case 3
                    x = chart.find('Name', [AuxVarMap.PREFIX_VARIABLE, 'tmax']);
                case 4
                    x = chart.find('Name', [AuxVarMap.PREFIX_VARIABLE, 'maxt']);
            end
            if (~ isempty(x))
                break;
            end
        end
        if (~ isempty(x))
            maxTime = str2double(x.Props.InitialValue);
        else
            'Error: not able to infer a maximum simulation time'
            return;
        end
    end
    
    % deactivate zero crossing detection
%     set_param(model, 'MaxConsecutiveZCsMsg', 'none');
    
    % initialize plot related data
    %figure();
    hold on; % activate several plots in one figure
    
    % set colors
    %colors = ['rgbcmyk'];
    %colors = ['gbcmk'];
    colors = ['k'];
    
    % set random seed to default for reproducing results
    if (seed_option)
        rng('default');
    end
    
    % set model parameters
    assignin('base', AuxVarMap.maxT, maxTime); % maximum simulation time
    assignin('base', AuxVarMap.eps, epsilon); % epsilon for equality-interval
    assignin('base', AuxVarMap.maxResets, max(maxResets, 1)); % # backtrackings
    
    % additional configuration
    %simulation_config(model,maxTime,1);
    
    for i = 1 : iterations
        if opt_time
            ['start iteration: ', num2str(i), ' at ', num2str(toc)]
        end
        
        % set model parameter: fresh random seed
        assignin('base', AuxVarMap.seed, randi(1e6));
        
         if opt_config_change
%             configSet_mdl = configSet.copy;
%             configSetNames = get_param(configSet_mdl, 'ObjectParameters');
% 
%             cfg_name = get_param(configSet_mdl, 'Name')
%             try
%                 cfg_max_timestep = get_param(configSet_mdl, 'MaxStep')
%                 set_param(configSet_mdl, 'MaxStep', num2str(rand(1,1) * maxTime / 1000));
%             end
%             set_param(configSet_mdl, 'Name', ['rand', cfg_name, num2str(i)]);
%             cfg_name = get_param(configSet_mdl, 'Name')
%             try
%                 cfg_max_timestep = get_param(configSet_mdl, 'MaxStep');
%             end
%             attachConfigSet(model, configSet_mdl, false);
%             setActiveConfigSet(model, cfg_name);
            
             simulation_config(model,maxTime,rand(1,1));
        end
        
        
        % simulate model
        [~] = sim(model);
        % problem with scopedata if we change parameters here; maybe set
        % active config set elsewhere before simulation loop? doesn't seem
        % to be a problem to set this in general (e.g., manually in the
        % model), just something strange with this particular command
        %[~] = sim(model, 'MaxStep', '1e-4');
        %[~] = sim(model, configSet);
        
        % read data from scope block
        [time, values, labels] = resultsToMatrix(ScopeData, filter, 1);
        
        % plot trajectories
        color = colors(1 + mod(i, length(colors)));
        if plot_option == plot_phase
            vec_il = values(2, :);
            vec_vc = values(3, :);
            hold on;
            %marker = '.';
            marker = 'o';
            marker_size = 15;
            scatter(vec_il,vec_vc,marker_size,marker,'filled', color);
            %scatter(vec_il,vec_vc,marker, color);
            %scatter(vec_il,vec_vc, color, 'filled');
        elseif (plot_option)
            NUM_PLOTS = size(labels, 2);
            if (i == 1)
                % initialize plot structure
                p = zeros(1, NUM_PLOTS);
                for j = 1 : NUM_PLOTS
                    p(j) = subplot(NUM_PLOTS, 1, j);
                    %xlabel(p(j), 'time');
                    %ylabel(p(j), labels{j});
                    hold(p(j), 'on');
                end
            end
            
            % plot each variable in a single diagram
            for j = 1 : NUM_PLOTS
                %plot(p(j), time', values(j, :)', color);
                marker = 'o';
                marker_size = 2;
                scatter(p(j), time', values(j, :)', marker_size,marker,'filled', color);
                %scatter(p(j), time', values(j, :)', color);
            end
        else
            % plot all variables in one diagram
            plot(time', values', color);
            %scatter(time', values', color);
            %xlabel('time');
        end
    end
    
    % deactivate several plots again
    hold off;
end
